# -*- coding: utf-8 -*-

from datetime import datetime
from random import shuffle
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib.auth.forms import AuthenticationForm
from django.db import models as db




# -*- coding: utf-8 -*-
import os, os.path

def publish(path):
    return os.sep.join((os.path.split(__file__)[0], path,))

current_dir_support = publish('current_dir_support.py')
namespace_package_support = publish('namespace_package_support.py')
profile_support = publish('profile_support.py')



"""
A set of type-converting filters.
"""
from django.template import Library

register = Library()


def numberify(obj, default=0):
    """
    Converts any type to ``int`` or returns a fallback number for falsy values. Except for
    falsy values, throws traditional ``ValueError`` if the input is uncovertable to ``int``.

    Template tag available in ``common`` app's ``converters`` library.
    
    :param obj: the object to convert

    :param default: a fallback ``int`` for falsy objects
    """

    return int(obj) if obj else default


def nullify(obj):
    """
    Converts any falsy value to ``None``.
 
    Template tag available in ``common`` app's ``converters`` library.
    """
    
    return obj if obj else None




"""
Contains a small set of useful abstract model base classes that are not application-specific.
"""

from datetime import datetime
from django.db import models as db
from django.utils.translation import ugettext_lazy as _
from django.core.mail import send_mail


class Named(db.Model):
    """
    Describes an abstract model with a ``name`` field.
    """
    name = db.CharField(verbose_name=_("name"), max_length=20, unique=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.name


class TimeTrackable(db.Model):
    """
    Describes an abstract model whose lifecycle is tracked by time. Includes
    a ``created`` field that is set automatically upon object creation and
    a ``modified`` field that is set automatically upon calling ``save()`` on
    the object.
    """
    created = db.DateTimeField(verbose_name=_("date created"), default=datetime.now)
    modified = db.DateTimeField(verbose_name=_("last modified"), default=datetime.now)
    
    class Meta:
        abstract = True

    def save(self):
        self.modified = datetime.now()
        super(TimeTrackable, self).save()


class DisplayCounter(db.Model):
    """
    Describes an abstract model which display count can be incremented by calling
    ``bump()``. Models inheriting from ``DisplayCounter`` can define a special ``bump_save()``
    method which is called instead of the default ``save()`` on each ``bump()`` (for instance
    to circumvent updating the ``modified`` field if the model is also ``TimeTrackable``.
    """
    display_count = db.PositiveIntegerField(verbose_name=_("display count"), default=0, editable=False)

    def bump(self):
        self.display_count += 1
        if not 'bump_save' in dir(self):
            self.save()
        else:
            self.bump_save()

    class Meta:
        abstract = True


class ViewableSoftDeletableManager(db.Manager):
    """
    An objet manager to automatically hide objects that were soft deleted for models
    inheriting ``SoftDeletable``.
    """
    def get_query_set(self):                                                                                                                                                                       
        # get the original query set
        query_set = super(ViewableSoftDeletableManager, self).get_query_set()
        # leave rows which are deleted
        query_set = query_set.filter(deleted=False)
        return query_set


class SoftDeletable(db.Model):
    """
    Describes an abstract models which can be soft deleted, that is instead of actually
    removing objects from the database, they have a ``deleted`` field which is set to ``True``
    and the object is then invisible in normal operations (thanks to ``ViewableSoftDeletableManager``).
    """
    deleted = db.BooleanField(verbose_name=_("deleted"), default=False)
    admin_objects = db.Manager()                                                                                                                                                                         
    objects = ViewableSoftDeletableManager()

    class Meta:
        abstract = True
    

# For now this needs to be at the end of the file.
# FIXME: move it where it's supposed to be.

import os
from time import sleep
from threading import Thread

class MassMailer(Thread):
    """
    A thread that can be used to mail the specified profiles with a certain message.
    After every message it waits a specified time (by default a second).
    """
    def __init__ (self, profiles, subject, content, inverval=1.0, force=False):
        """ Creates the thread.

        :param profiles: a sequence of profiles that are to be mailed

        :param subject: the subject of the message to be sent

        :param content: the actual content to be sent
        
        :param interval: number of seconds to wait after sending every message

        :param force: if ``True``, privacy settings of the users are disregarded
        """

        Thread.__init__(self)
        self.profiles = profiles
        self.subject = subject
        self.content = content
        self.interval = interval
        self.force = force
        
    def run(self):
        print "Mailer subprocess started (%d)." % os.getpid()
        for profile in self.profiles:
            mail = profile.user.email
            # FIXME: check privacy
            send_mail(self.subject, self.content, None, [mail])
            print "Mailer subprocess (%d): sent mail to %s." % (os.getpid(), mail)
            sleep(self.interval)

# NO CODE BEYOND THIS POINT



# -*- coding: utf-8 -*-
### NOTE:
### This snippet will be executed in the ``settings.py`` context and is not an actual
### Python module that you would import in a traditional sense. When you do import its
### name, you get the path to this file so you can ``execfile()`` it.

if 'CURRENT_DIR' not in locals():
    raise ValueError, "langacore.kit.django.profile_support requires current_dir_support."

import os, os.path
import sys
from optparse import make_option
import django.core.management.base
from django.core.management.base import BaseCommand
from django.core.management.base import handle_default_options as original_default_options 

local_profile = os.environ.get('DJANGO_SETTINGS_PROFILE', 'local')

if SETTINGS_PATH_MODE == 'flat': 
    local_settings = '%s-%s.py' % (SETTINGS_PATH_PREFIX, local_profile)
elif SETTINGS_PATH_MODE == 'nested': 
    local_settings = '%s%s%s.py' % (SETTINGS_PATH_PREFIX, os.sep, local_profile)
else:
    raise ValueError, "Unsupported settings path mode '%s'" % SETTINGS_PATH_MODE

if os.path.exists(local_settings):
    execfile(local_settings)
elif local_profile != 'local':
    raise ValueError, "%s does not exist." % local_settings

def nothing(*args, **kwargs): pass
    
import django.core.mail
#django.core.mail.send_mail = nothing

if not hasattr(BaseCommand, '_langacore_patched'):
    BaseCommand.option_list = BaseCommand.option_list + (make_option('--profile',
                        help='Name of a settings module, e.g. "debug", "syncdb", etc. If this isn\'t provided, "local" profile is assumed.'),)
    BaseCommand._langacore_patched = True

    def handle_default_options(options):
        if options.profile:
            os.environ['DJANGO_SETTINGS_PROFILE'] = options.profile
        original_default_options(options)
    
    django.core.management.base.handle_default_options = handle_default_options



"""
Albeit useful, this module is still somewhat a mess in a really early state of development. Beware, there be dragons.
"""

import datetime
import re

from django import forms
from django.forms.widgets import Select
from django.forms.extras.widgets import RE_DATE, SelectDateWidget
from django.utils.dates import MONTHS
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _


GENDER_CHOICES = ((1, _("Female")),
                  (2, _("Male")),
                  (3, _("any")),
                 )

ANY_GENDER = 3


SEARCH_GAME_CHOICES = (('cs16', _("Counter-Strike 1.6")),
                       ('cod4', _("Call of Duty 4")),
                       ('any', _("any")),
                      )


ANY_GAME = 'any'


ACCOUNT_CHOICES = ((1, _("complete")),
                   (2, _("incomplete")),
                   (3, _("any account")),
                  )


ANY_ACCOUNT = 3
COMPLETE_ACCOUNT = 1
INCOMPLETE_ACCOUNT = 2

class PolishSelectDateWidget(SelectDateWidget):
    def __init__(self, attrs=None, years=None):
        super(PolishSelectDateWidget, self).__init__(attrs, years)

    def render(self, name, value, attrs=None):
        try:
            year_val, month_val, day_val = value.year, value.month, value.day
        except AttributeError:
            year_val = month_val = day_val = None
            if isinstance(value, basestring):
                match = RE_DATE.match(value)
                if match:
                    year_val, month_val, day_val = [int(v) for v in match.groups()]

        output = []

        if 'id' in self.attrs:
            id_ = self.attrs['id']
        else:
            id_ = 'id_%s' % name

        day_choices = [(i, i) for i in range(1, 32)]
        local_attrs = self.build_attrs(id=self.day_field % id_)
        select_html = Select(choices=day_choices).render(self.day_field % name, day_val, local_attrs)
        output.append(select_html)

        month_choices = MONTHS.items()
        month_choices.sort()
        local_attrs['id'] = self.month_field % id_
        select_html = Select(choices=month_choices).render(self.month_field % name, month_val, local_attrs)
        output.append(select_html)

        year_choices = [(i, i) for i in self.years]
        local_attrs['id'] = self.year_field % id_
        select_html = Select(choices=year_choices).render(self.year_field % name, year_val, local_attrs)
        output.append(select_html)

        return mark_safe(u'\n'.join(output))


class MassMailingForm(forms.Form):
    gender = forms.ChoiceField(label=_("Gender"), choices=GENDER_CHOICES, initial=ANY_GENDER)
    game = forms.ChoiceField(label=_("Game"), choices=SEARCH_GAME_CHOICES, initial=ANY_GAME)
    account = forms.ChoiceField(label=_("Account"), choices=ACCOUNT_CHOICES, initial=ANY_ACCOUNT, help_text=_("Incomplete accounts don't have Allegro login or Steam ID/PB GUID filled."))
    subject = forms.CharField(label=_("Subject"), max_length=150, required=False)
    content = forms.CharField(label=_("Content"), widget=forms.Textarea)
    force_privacy = forms.BooleanField(label=_("Ignore privacy"), initial=False, required=False)

MassMailingForm.base_fields['force_privacy'].reverse = True



"""A drop-in substitute for the default ``django.core.cache`` object. This implementation
is using a modified mint cache approach based on http://www.djangosnippets.org/snippets/793/.

This implementation does not support specifying fallback values for the ``get()`` function since
it would not be possible to otherwise reliably communicate that the value is *stale* and has
to be updated.

Configured by three values in ``settings.py``:

* ``CACHE_DEFAULT_TIMEOUT`` - how long a set value should be considered valid (in seconds).
                              After this period the value is considered *stale*, a single request
                              starts to update it, and subsequent requests get the old value until
                              the value gets updated. **Default**: 300 seconds.

* ``CACHE_FILELOCK_PATH`` - path to a non-existant file which will work as an interprocess lock to
                            make cache access atomic. **Default**: ``/tmp/langacore_django_cache.lock``

* ``CACHE_MINT_DELAY`` - an upper bound on how long a value should take to be generated (in seconds).
                         This value is used for *stale* keys and is the real time after which the
                         key is completely removed from the cache. **Default**: 30 seconds.
"""

# More info in the documentation at http://packages.python.org/langacore.kit.django/

import time
from threading import RLock

from django.core.cache import cache
from django.conf import settings

from langacore.kit.concurrency import synchronized

CACHE_MINT_DELAY = getattr(settings, 'CACHE_MINT_DELAY', 30)
CACHE_DEFAULT_TIMEOUT = getattr(settings, 'CACHE_DEFAULT_TIMEOUT', 300)
CACHE_FILELOCK_PATH = getattr(settings, 'CACHE_FILELOCK_PATH', '/tmp/langacore_django_cache.lock')


@synchronized(path=CACHE_FILELOCK_PATH)
def get(key):
    """
    Get a value from the cache. 

    :param key: the key for which to return the value
    """

    packed_val = cache.get(key)
    if packed_val is None:
        return None 
    val, refresh_time, is_stale = packed_val
    if (time.time() > refresh_time) and not is_stale:
        # Store the stale value while the cache revalidates for another
        # CACHE_MINT_DELAY seconds.
        set(key, val, timeout=CACHE_MINT_DELAY, _is_stale=True)
        return None
    return val


@synchronized(path=CACHE_FILELOCK_PATH)
def set(key, val, timeout=CACHE_DEFAULT_TIMEOUT, _is_stale=False):
    """
    Set a value in the cache.

    :param key: the key under which to set the value

    :param val: the value to set

    :param timeout: how long should this value be valid, by default CACHE_DEFAULT_TIMEOUT

    :param _is_stale: boolean, used internally to set a stale value in the cache back-end. Don't use on your own. 
    """
    refresh_time = timeout + time.time()
    # if not stale, add the mint delay to the actual refresh so we can have
    # the value stored a bit longer in the backend than it would be otherwise
    real_refresh = timeout if _is_stale else timeout + CACHE_MINT_DELAY
    packed_val = (val, refresh_time, _is_stale)
    return cache.set(key, packed_val, real_refresh)


@synchronized(path=CACHE_FILELOCK_PATH)
def delete(key):
    """
    Removes a value from the cache.

    :param key: the key to delete from the cache
    """

    return cache.delete(key)



"""
A set of common independent filters usable directly in Python code.
"""

from langacore.kit.django.common.templatetags.converters import numberify, nullify 
from langacore.kit.django.common.templatetags.strings import strike_empty, slugify, title, transliterate



# -*- coding: utf-8 -*-
### NOTE:
### This snippet will be executed in the ``settings.py`` context and is not an actual
### Python module that you would import in a traditional sense. When you do import its
### name, you get the path to this file so you can ``execfile()`` it.

import os.path

def _get_settings_path_mode(path):
    if os.path.isdir(path):
        return 'nested'
    elif os.path.exists(path + '.py'):
        return 'flat'
    else:
        return None


CURRENT_DIR = os.path.realpath(os.path.dirname(__file__)) + os.path.sep
_django_settings_dir = os.environ.get('DJANGO_SETTINGS_DIR', CURRENT_DIR)

SETTINGS_PATH_PREFIX = os.path.join(_django_settings_dir, 'settings')
SETTINGS_PATH_MODE = _get_settings_path_mode(SETTINGS_PATH_PREFIX)

if not SETTINGS_PATH_MODE:
    # no settings dir/settings.py file in DJANGO_SETTINGS_DIR or the current directory
    # let's try the parent:
    CURRENT_DIR = os.path.realpath(os.path.dirname(__file__) + os.path.sep + '..') + os.path.sep
    SETTINGS_PATH_PREFIX = os.path.join(_django_settings_dir, '..', 'settings')
    SETTINGS_PATH_MODE = _get_settings_path_mode(SETTINGS_PATH_PREFIX)
      
if not SETTINGS_PATH_MODE:
    raise ValueError, "Django settings structure not recognized."



# -*- coding: utf-8 -*-
"""
A set of string-related filters.
"""

import re
from django.template import Library
from django.template.defaultfilters import slugify as crippled_slugify

from langacore.kit.i18n import translit

register = Library()


@register.filter
def title(text):
    """
    Like built-in \\|title filter but leaves existing caps alone, eg.
    ``'I work for the FBI'`` -> ``'I Work For The FBI'``.

    Template tag available in ``common`` app's ``strings`` library.
    """
    pattern=re.compile(r"(\s|^)\w")
    cap=lambda x: x.group().upper()

    return pattern.sub(cap, text + u'') #+ u'' to handle proxy objects


@register.filter
def transliterate(text, country_code, text_lang=''):
    """
    If the country with the specified code is using Cyrillic alphabet,
    transliterates Latin strings using ``langacore.kit.i18n.translit``.

    Template tag available in ``common`` app's ``strings`` library.
    """
    return translit.any(country_code, text, input_lang=text_lang)


@register.filter
def slugify(text, fallback='untitled-0'):
    """
    A version of the builtin ``django.template.defaultfilters`` which doesn't let empty
    strings as outcomes.

    Template tag available in ``common`` app's ``strings`` library.
    
    :param text: the text to slugify

    :param fallback: the text to return in case the builtin slugify returns an empty value
    """
 
    slug = crippled_slugify(text)
    return slug if slug else fallback


@register.filter
def strike_empty(obj):
    """
    Converts any falsy value to ``u"---"``.
    
    Template tag available in ``common`` app's ``strings`` library.
    """

    return obj if obj else u"---"



# -*- coding: utf-8 -*-
from django.conf import settings
from django.template import RequestContext
from django.http import HttpResponseRedirect
import django.shortcuts


def redirect(request, link='/', chain=False):
    context = RequestContext(request, {})
    if 'auth__redirect_to' in context:
        link = context['auth__redirect_to']

    if chain:
        if link.rfind('?') != -1:
            link = link + '&redirect_to=%s' % link[:link.rfind('?')]
        else:
            link = link + '?redirect_to=%s' % link 

    return HttpResponseRedirect(link)


def render(request, template_name, context, debug=False):
    if hasattr(settings, 'AUTH_PROFILE_MODULE'):
        if 'user_profile' in context:
            raise KeyError, "langacore.kit.django.helpers.render() doesn't accept contexts with 'user_profile'"
        if 'other_user_profile' in context:
            raise KeyError, "langacore.kit.django.helpers.render() doesn't accept contexts with 'other_user_profile'"
    
        context['user_profile'] = request.user.get_profile() if request.user.is_authenticated() else None 
        context['other_user_profile'] = context['other_user'].get_profile() if 'other_user' in context else None

    if debug:
        for key in context:
            print "context['%s'] = %s (type: %s) (repr: %s)" % (key, str(context[key]), type(context[key]), repr(context[key]))
    return django.shortcuts.render_to_response(template_name, RequestContext(request, context))


class Choice(object):
    global_id = 0

    def __init__(self, description, id=-255):
        self.id = id
        self.desc = description
        self.global_id = Choice.global_id
        Choice.global_id += 1


class Choices(list):
    Choice = Choice

    def __init__(self):
        values = []

        for k, v in self.__class__.__dict__.items():
            if type(v) == Choice:
                 values.append(v)

        if not values:
            raise ValueError("Choices class declared with no actual choice fields.")

        values.sort(lambda x, y: x.global_id - y.global_id)

        last_choice_id = 0
        for choice in values:
            if choice.id == -255:
                last_choice_id += 1
                choice.id = last_choice_id
            last_choice_id = choice.id
            self.append((choice.id, choice.desc))


if __name__ == '__main__':
    # crude test
    class Colors(Choices):
        _ = Choices.Choice

        white = _("White")
        yellow = _("Yellow")
        red = _("Red")
        green = _("Green")
        black = _("Black")

    print Colors()
    print Colors.white.id, Colors.white.desc

    class NoChoice(Choices):
        not_a_choice = "Not a Choice() object"

    print NoChoice()



# -*- coding: utf-8 -*-

"""
BBCode-related template tags.
"""
from django.template import Library
from django.utils.safestring import mark_safe
from postmarkup import render_bbcode

register = Library()

@register.filter
def bbcode(text):
    """
    Simply encode the BBCode within the text.

    Template tag available in ``common`` app's ``bbcode`` library.
    
    :param text: the string to render as BBCode
    """
    return mark_safe(render_bbcode(text))
